/*
 * A system pager implementation residing in a separate task.
 *
 * Copyright (C) 2022, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <vector>

#include <exec/elf.h>
#include <exec/pager.h>
#include <exec/segment.h>
#include <fsserver/notification.h>
#include <resource/resource.h>

#include "parent_pager_object_interface.h"



/* A simple system pager also acting as a region mapper. */

class ExternalPager : public ExecPager, public ParentPagerObject,
                      public NotificationSupport, public Resource
{
protected:

  /* Resources associated with the created process. */

  l4_cap_idx_t _task = L4_INVALID_CAP, _mapped_task = L4_INVALID_CAP,
               _parent = L4_INVALID_CAP, _mapped_parent = L4_INVALID_CAP,
               _pager = L4_INVALID_CAP, _mapped_pager = L4_INVALID_CAP,
               _ipc_gate = L4_INVALID_CAP;
  std::vector<l4_cap_idx_t> _threads, _mapped_threads;

  /* Resources supporting the internal pager. */

  ExplicitSegment *_rm_stack = NULL;
  Payload *_rm_payload = NULL;

public:
  explicit ExternalPager(address_t start = 0, address_t end = 0);

  /* Server details. */

  virtual ipc_server_default_config_type config();

  virtual void *interface()
  { return static_cast<ParentPagerObject *>(this); }

  /* Task and thread management. */

  virtual void add_thread(l4_cap_idx_t cap, l4_cap_idx_t mapped_cap);
  virtual void set_gate(l4_cap_idx_t cap);
  virtual void set_pager(l4_cap_idx_t cap, l4_cap_idx_t mapped_cap);
  virtual void set_parent(l4_cap_idx_t cap, l4_cap_idx_t mapped_cap);
  virtual void set_task(l4_cap_idx_t cap, l4_cap_idx_t mapped_cap);

  /* Internal pager resource management. */

  virtual void set_payload(Payload *payload);
  virtual void set_stack(ExplicitSegment *stack);

  /* Resource methods. */

  virtual void close();

  /* Notification methods, implementing PagerObject. */

  virtual long exception(l4_exc_regs_t regs,
                         l4_snd_fpage_t *region);

  virtual long page_fault(l4_umword_t pfa, l4_umword_t pc,
                          l4_snd_fpage_t *region);

  /* Region manager/mapper methods. */

  virtual long attach(address_t *start, address_t size, map_flags_t flags,
                      l4_cap_idx_t ds, address_t offset, unsigned char align);

  /* Parent methods. */

  virtual long signal(unsigned long sig, unsigned long val);

  /* Notification methods. */

  virtual long subscribe(l4_cap_idx_t notifier, notify_flags_t flags);

  virtual long unsubscribe(l4_cap_idx_t notifier);
};

/* vim: tabstop=2 expandtab shiftwidth=2
*/
